@@ -42,9 +42,11 @@ class HistoryEventHandler(AbstractControllerSubscriber):
 
             # Create a new scene history writer with each new scene (config
             # data) so we always create a new, separate scene history file.
+            # Also, ensure previous history item is cleared.
             self.__history_writer = HistoryWriter(payload.scene_config,
                                                   hist_info,
                                                   payload.timestamp)
+            self.__history_item = None
 
     def on_before_step(self, payload: BeforeStepPayload):
         if payload.config.is_history_enabled():
@@ -138,11 +140,28 @@ class HistoryWriter(object):
         if self.scene_history_file:
             logger.info(f"Saving history file {self.scene_history_file}")
             with open(self.scene_history_file, "a+") as history_file:
+                history = self._round_all(self.history_obj)
                 history_file.write(
                     json.dumps(
-                        self.history_obj,
+                        history,
                         cls=NumpyEncoder))
 
+    # This could easily be moved to a more accessible utility file.
+    def _round_all(self, obj, num_digits=4):
+        if isinstance(obj, float):
+            obj = round(obj, num_digits)
+        elif isinstance(obj, list):
+            obj = [self._round_all(item) for item in obj]
+        elif isinstance(obj, dict):
+            new_obj = {
+                subkey: self._round_all(val) for subkey,
+                val in obj.items()}
+            obj = new_obj
+        elif isinstance(obj, tuple):
+            obj = list(obj)
+            obj = tuple(self._round_all(obj))
+        return obj
+
     def update_history_output(
             self,
             history: SceneHistory) -> SceneHistory:
