@@ -10,7 +10,7 @@ import cv2
 import numpy as np
 import PIL.Image
 import skimage.draw
-from shapely import geometry
+from shapely import affinity, geometry
 
 from machine_common_sense.config_manager import SceneConfiguration, Vector3d
 
@@ -53,11 +53,11 @@ class XZHeading():
 
 @dataclass
 class Robot():
-    '''AI robot/performer location and heading in Unity'''
+    '''AI robot/performer location and rotation in Unity'''
     x: float
     y: float
     z: float
-    heading: XZHeading
+    rotation_y: float
 
 
 @dataclass
@@ -155,12 +155,11 @@ class TopDownPlotter():
     BORDER_COLOR = colour.COLOR_NAME_TO_RGB['white']
     CENTER_COLOR = colour.COLOR_NAME_TO_RGB['gray']
     GRID_COLOR = colour.COLOR_NAME_TO_RGB['darkslategray']
-    ROBOT_COLOR = colour.COLOR_NAME_TO_RGB['red']
-    GOAL_COLOR = colour.COLOR_NAME_TO_RGB['gold']
+    ROBOT_COLOR = colour.COLOR_NAME_TO_RGB['blue']
+    DEFAULT_GOAL_COLOR = colour.COLOR_NAME_TO_RGB['lime']
 
-    ROBOT_PLOT_WIDTH = 0.2
-    HEADING_LENGTH = 0.2
-    ROBOT_NOSE_RADIUS = 0.08
+    OBJECT_INSIDE_PERCENTAGE = 0.5
+    HEADING_LENGTH = 0.6
     PLOT_IMAGE_SIZE = 512
     UNIT_CELL_WIDTH = 1
 
@@ -222,7 +221,7 @@ class TopDownPlotter():
             scene_config.room_dimensions)
         img = self._draw_lava(
             img,
-            scene_config.lava,
+            scene_config.retrieve_lava(),
             scene_config.room_dimensions)
 
         return img
@@ -254,6 +253,14 @@ class TopDownPlotter():
             y=int(self.image_center.y - scene_pt.z * self.scale.y)
         )
 
+    def _convert_points(self, points: list) -> tuple:
+        # convert list of tuples to list of rows and list of columns
+        cs, rs = map(list, zip(*points))
+        # convert room coordinates to image coordinates
+        rs = list(map(lambda r: self.image_center.y - r * self.scale.y, rs))
+        cs = list(map(lambda c: self.image_center.x + c * self.scale.x, cs))
+        return cs, rs
+
     def _draw_step_number(self, img: np.ndarray,
                           step_number: int) -> np.ndarray:
         '''Write the step number on the plot image'''
@@ -416,16 +423,16 @@ class TopDownPlotter():
             return img
 
         for area in lava:
-            area_center = SceneCoord(x=area.x, y=0, z=area.z)
-            half_cell_pos = SceneCoord(
-                self.UNIT_CELL_WIDTH / 2, 0, self.UNIT_CELL_WIDTH / 2)
+            area_pos_ul = SceneCoord(x=area[0], y=0, z=area[1])
+            area_pos_lr = SceneCoord(x=area[2], y=0, z=area[3])
+            midpoint = area_pos_ul | area_pos_lr
             size_reducer_left, size_reducer_right = (
-                self._plotter_hole_texture_size_reducer(
-                    area, room_dim))
+                self._plotter_hole_texture_size_reducer(midpoint, room_dim)
+            )
             if size_reducer_left is None:
                 continue
-            area_pos_ul = area_center - half_cell_pos + size_reducer_left
-            area_pos_lr = area_center + half_cell_pos + size_reducer_right
+            area_pos_ul = area_pos_ul + size_reducer_left
+            area_pos_lr = area_pos_lr + size_reducer_right
             area_img_pos_ul = self._convert_to_image_coords(area_pos_ul)
             area_img_pos_lr = self._convert_to_image_coords(area_pos_lr)
 
@@ -542,12 +549,11 @@ class TopDownPlotter():
         return PIL.Image.fromarray(img, "RGB")
 
     def _draw_robot(self, img: np.ndarray, robot_metadata: Dict) -> np.ndarray:
-        '''Plot the robot position and heading'''
+        '''Plot the robot triangle'''
         if robot_metadata is None:
             return img
         robot = self._create_robot(robot_metadata)
         img = self._draw_robot_position(img, robot)
-        img = self._draw_robot_heading(img, robot)
         return img
 
     def _create_robot(self, robot_metadata: Dict) -> Robot:
@@ -563,50 +569,43 @@ class TopDownPlotter():
             z = 0.0
         rotation = robot_metadata.get('rotation')
         rotation_y = rotation.get('y') if rotation is not None else 0.0
-        heading = self._calculate_heading(
-            rotation_angle=360.0 - rotation_y,
-            heading_length=self.HEADING_LENGTH
-        )
-        return Robot(x, y, z, heading)
+        return Robot(x, y, z, 360 - rotation_y)
 
     def _calculate_heading(self, rotation_angle: float,
                            heading_length: float) -> XZHeading:
         '''Calculate XZ heading vector from the rotation angle'''
         s = math.sin(math.radians(rotation_angle))
         c = math.cos(math.radians(rotation_angle))
-        vec_x = 0 * c - heading_length * s
-        vec_z = 0 * s + heading_length * c
+        vec_x = heading_length * s * -1
+        vec_z = heading_length * c
         return XZHeading(vec_x, vec_z)
 
     def _draw_robot_position(self, img: np.ndarray,
                              robot: Robot) -> np.ndarray:
-        '''Draw the robot's scene XZ position in the plot'''
-        robot_position = self._convert_to_image_coords(
-            scene_pt=SceneCoord(robot.x, robot.y, robot.z)
-        )
-        rr, cc = skimage.draw.disk(
-            center=(
-                robot_position.y,
-                robot_position.x
-            ),
-            radius=self.ROBOT_PLOT_WIDTH * self.scale.x,
-            shape=img.shape[:2])
-        img[rr, cc] = self.ROBOT_COLOR
-        return img
-
-    def _draw_robot_heading(self, img: np.ndarray, robot: Robot) -> np.ndarray:
-        '''Draw the heading vector starting from the robot XZ position'''
-        y_point = int(self.image_center.y - robot.z * self.scale.y)
-        x_point = int(self.image_center.x + robot.x * self.scale.x)
-        rr, cc = skimage.draw.disk(
-            center=(
-                y_point - int(robot.heading.z * self.scale.x),
-                x_point + int(robot.heading.x * self.scale.x)
-            ),
-            radius=self.ROBOT_NOSE_RADIUS * self.scale.x,
-            shape=img.shape[:2]
-        )
-        img[rr, cc] = self.ROBOT_COLOR
+        '''Draw the robot's scene XZ position as a triangle in the plot'''
+        opposite_color = self._find_opposite_color(self.ROBOT_COLOR)
+        half_length = self.HEADING_LENGTH / 2.0
+        # Create the triangle points.
+        front = self._calculate_heading(robot.rotation_y, self.HEADING_LENGTH)
+        right = self._calculate_heading(120 + robot.rotation_y, half_length)
+        left = self._calculate_heading(240 + robot.rotation_y, half_length)
+        triangle = [(front.x, front.z), (right.x, right.z), (left.x, left.z)]
+        # Create the triangle polygon.
+        outer = geometry.MultiPoint(triangle).convex_hull
+        # Move the triangle from the origin to the correct position.
+        outer = affinity.translate(outer, robot.x, robot.z)
+        # Draw the robot as two triangles: first a large outer triangle of the
+        # opposite color as a border; and a small inner triangle as the center.
+        percent = self.OBJECT_INSIDE_PERCENTAGE
+        inner = affinity.scale(outer, xfact=percent, yfact=percent)
+        for polygon, color in [
+            (outer, opposite_color),
+            (inner, self.ROBOT_COLOR)
+        ]:
+            points = list(polygon.exterior.coords)
+            cs, rs = self._convert_points(points)
+            rr, cc = skimage.draw.polygon(rs, cs, shape=img.shape[:2])
+            img[rr, cc] = color
         return img
 
     def _find_plottable_objects(
@@ -637,9 +636,10 @@ class TopDownPlotter():
         for o in objects:
             obj = self._create_asset(o)
             if obj.bounds is not None:
-                img = self._draw_object(img, obj)
                 if goal_id is not None and o['objectId'] == goal_id:
                     img = self._draw_goal(img, obj)
+                else:
+                    img = self._draw_object(img, obj)
         return img
 
     def _create_asset(self, object_metadata: Dict) -> SceneAsset:
@@ -671,43 +671,52 @@ class TopDownPlotter():
         '''Convert color string to string'''
         if not color:
             color = 'ivory'
-
+        # Don't use the same color as the background (black).
         if color == 'black':
             color = 'ivory'
+        # Rose is apparently not a valid web color.
         if color == 'rose':
             color = 'deeppink'
+        # Azure as a web color is very different from the normal azure.
+        if color == 'azure':
+            color = 'deepskyblue'
         return color
 
     def _draw_object(self, img: np.ndarray,
-                     obj: SceneAsset) -> np.ndarray:
+                     obj: SceneAsset, is_goal: bool = False) -> np.ndarray:
         '''Draw the scene object'''
 
         obj_pts = [(pt.x, pt.z) for pt in obj.bounds.points]
-        polygon = geometry.MultiPoint(
-            obj_pts).convex_hull
+        polygon = geometry.MultiPoint(obj_pts).convex_hull
+        # Shrink the polygon for the goal object, since it will be drawn on top
+        # of the correctly-sized polygon from _draw_goal.
+        if is_goal:
+            percent = self.OBJECT_INSIDE_PERCENTAGE
+            polygon = affinity.scale(polygon, xfact=percent, yfact=percent)
         pts = polygon.exterior.coords
 
-        # convert list of tuples to list of rows and list of columns
-        cs, rs = map(list, zip(*pts))
-        # convert room coordinates to image coordinates
-        rs = list(map(lambda r: self.image_center.y - r * self.scale.y, rs))
-        cs = list(map(lambda c: self.image_center.x + c * self.scale.x, cs))
+        cs, rs = self._convert_points(pts)
+
+        # use dictionary get method for color retrieval
+        # if no match for that color string, then resort to the default color
+        clr = colour.COLOR_NAME_TO_RGB.get(
+            obj.color.lower(), self.DEFAULT_COLOR)
 
         # draw filled polygon if visible to the robot
-        if(obj.visible):
+        if is_goal or obj.visible:
             rr, cc = skimage.draw.polygon(
                 rs,
                 cs,
                 shape=img.shape[:2])
+            # Draw a small black polygon for a hidden goal object, since it
+            # already has a border around it.
+            if not obj.visible:
+                clr = self.BACKGROUND_COLOR
         else:
             rr, cc = skimage.draw.polygon_perimeter(
                 rs,
                 cs,
                 shape=img.shape[:2])
-        # use dictionary get method for color retrieval
-        # if no match for that color string, then resort to the default color
-        clr = colour.COLOR_NAME_TO_RGB.get(
-            obj.color.lower(), self.DEFAULT_COLOR)
         img[rr, cc] = clr
 
         # using ramp string prefix assumpation to make ramp determination
@@ -752,16 +761,28 @@ class TopDownPlotter():
             obj_pts).convex_hull
         pts = polygon.exterior.coords
 
-        cs, rs = map(list, zip(*pts))
-        # convert room coordinates to image coordinates
-        rs = list(map(lambda r: self.image_center.y - r * self.scale.y, rs))
-        cs = list(map(lambda c: self.image_center.x + c * self.scale.x, cs))
-        rr, cc = skimage.draw.polygon_perimeter(
-            rs,
-            cs,
-            shape=img.shape[:2])
-        img[rr, cc] = self.GOAL_COLOR
-        return img
+        color = colour.COLOR_NAME_TO_RGB.get(
+            obj.color.lower(),
+            self.DEFAULT_COLOR
+        )
+        opposite_color = self._find_opposite_color(color)
+        # If the object's color is greyscale, use a default instead.
+        if color[0] == color[1] == color[2]:
+            opposite_color = self.DEFAULT_GOAL_COLOR
+
+        cs, rs = self._convert_points(pts)
+
+        # Draw a correctly-sized polygon with the opposite of the goal's color.
+        # This will create a thick border around the 2nd polygon (see below).
+        rr, cc = skimage.draw.polygon(rs, cs, shape=img.shape[:2])
+        img[rr, cc] = opposite_color
+
+        # Then draw a smaller polygon on top of the 1st polygon in its middle.
+        return self._draw_object(img, obj, is_goal=True)
+
+    def _find_opposite_color(self, color: tuple) -> tuple:
+        '''Return the exact opposite of the given color.'''
+        return (255 - color[0], 255 - color[1], 255 - color[2])
 
 
 if __name__ == '__main__':
